
var Relax = function()
{
	var relax = {};
	
	relax.VERSION = '1.5';

	
	relax.checkIsAdmin = function(message, callback, servletDir)
	{
		var url = (servletDir || '../') + 'Session.servlet?ID='+App.getSessionId();
		url += '&cmd=CheckIsAdmin';
		if (message) url += '&message='+encodeURIComponent(message);
		if (!callback) callback = function() {};
		Wizard.asyncJsonRequest(url, callback);
	}
	
	return relax;
}();


var Wizard = function()
{
	var wizard = {};
	var internal = {};
	var debug = 0;
	
	var numLiveAsyncRequests = 0;
	var hasFatalError = false;
	var currentStep = 1;
	var hasSentRegistration = false;
	var forceConfirm = false;
	var noConfirmOnFirstStep = true;
	var progressTimer = 0;
	
	/**
		Submit an ansynchronous request for JSON data. 'GET' method
		is used if 'method' parameter is not set.
		The 'callback' should be a method accepting a single
		JSON parameter object. If the 'status' of the response
		is not 'ok' this method will call setFatalError and
		without calling the callback method.
		
		The 'errorCallback' is an optional function parameter that should
		take two parameters. The first is an Error object and the second is
		the JSON response. The seconds parameter only has a value if the error 
		happens after parsing the response and is null otherwise.
	*/
	wizard.asyncJsonRequest = function(url, callback, method, postdata, errorCallback)
	{
		if (debug)
		{
			App.debug((method || 'GET') + ': ' + url);
			if (postdata) App.debug(postdata);
		}
		var request = Ajax.getXmlHttpRequest();
		request.jsonCallback = callback;
		request.errorCallback = errorCallback;
		request.open(method || 'GET', url, true);
		// Firefox only adds request header with needed "boundary" info, if request header is not set explicitly
		if (postdata && !(postdata instanceof FormData)) 
		{
			request.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
		}
		Ajax.setReadyStateHandler(request, internal.asyncCallbackWrapper, internal.asyncCallbackWrapper);
		numLiveAsyncRequests++;
		request.send(postdata || null);
	}
	
	/**
		Callback wrapper for ansynchronous requests.
	*/
	internal.asyncCallbackWrapper = function(request)
	{
		numLiveAsyncRequests--;
		if (debug) App.debug(request.responseText);
		if (!hasFatalError) Wizard.hideWizardStatus();

		var response = null;
		var messages = null;
		try
		{
			if (request.status != 200)
			{
				throw new Error(request.responseText);
			}
			else
			{
				if (request.getResponseHeader('Content-Type').indexOf('json') != -1)
				{
					response = JSON.parse(request.responseText);
					messages = response.messages;
					if (response.status != 'ok')
					{
						throw new Error(response.message || response.stacktrace);
					}
				}
				else
				{
					response = request.responseText;
				}
				if (request.jsonCallback && request.jsonCallback.call)
				{
					request.jsonCallback.call(null, response);
				}
			}
		}
		catch (err)
		{
			if (request.errorCallback && request.errorCallback.call)
			{
				request.errorCallback.call(null, err, response);
			}
			else
			{
				Wizard.setFatalError(err, messages);
			}
		}
	}
	
	wizard.isValid = function(element)
	{
		return Data.int(element, 'valid', 1);
	}

	wizard.initFileSelectionField = function(element)
	{
		element = Doc.element(element);
		Events.addEventHandler(element, 'change', wizard.fileChanged);
		Events.sendChangeEvent(element);
	}
	
	wizard.fileChanged = function(event)
	{
		var target = event.currentTarget;
		
		var isValid = false;
		var isWarning = false;
		
		// Reset current status
		wizard.setInputStatus(target);
		
		var path = target.value;
		if (path == '')
		{
			// No file, is this an error or a warning?
			var optional = target.className.indexOf('required') == -1;
			wizard.setInputStatus(target, optional ? 'warning' : 'invalid', 'Missing');
			isValid = optional;
			isWarning = optional;
		}
		else
		{
			Wizard.setInputStatus(target, 'valid');
			isValid = true;
		}
		
		Data.set(target, 'valid', isValid ? 1 : 0);
		Data.set(target, 'warning', isWarning ? 1 : 0);
		
		if (!isValid && target.focus) target.focus();
	}
	
	/**
		Change display status for an input field.
		@param prefix ID prefix to locate tags
		@param clazz The message clazz (valid, invalid, warning) or empty
		@param message The message (may be empty)
	*/
	wizard.setInputStatus = function(prefix, clazz, message)
	{
		if (prefix.id) prefix = prefix.id;
		
		var statusTag = Doc.element(prefix + '.status');
		
		if (statusTag)
		{
			statusTag.className = 'status ' + (clazz || '');
			var trTag = statusTag.parentNode;
			var trClass = Data.get(trTag, 'original-class');
			if (!trClass)
			{
				trClass = trTag.className || 'foo';
				Data.set(trTag, 'original-class', trClass);
			}
			trTag.className = trClass + ' ' + (clazz || '');
		}
		
		var msgTag = Doc.element(prefix + '.message');
		if (msgTag)
		{
			if (message)
			{
				msgTag.innerHTML = message;
				Doc.show(msgTag, Data.get(msgTag, 'display', 'inline'));
			}
			else
			{
				msgTag.innerHTML = message;
				Doc.hide(msgTag);
			}
		}
		else if (statusTag)
		{
			statusTag.title = message || '';
		}
	}
	
	/**
		Event handler for input fields, that go to the
		next wizard step if TAB or ENTER key is pressed.
		A short delay is used to allow other event handler
		to run first (eg. for validation).
	*/
	wizard.goNextOnTabOrEnter = function(event)
	{
		if (event.keyCode == 9 || event.keyCode == 13) 
		{
			setTimeout(function() { wizard.goNext(true) }, 200);
		}
	}
	
	/**
		Event handler for input fields, that go to the
		next wizard step if TAB key is pressed.
		A short delay is used to allow other event handler
		to run first (eg. for validation).
	*/
	wizard.goNextOnTab = function(event)
	{
		if (event.keyCode == 9) 
		{
			setTimeout(function() { wizard.goNext(true) }, 200);
		}
	}

	/**
		Event handler for input fields, that changes
		focus to the next element. The next element
		should be defined by 'data-next-focus' attribute.
		It is recommended that this is consisistent with TAB.
	*/
	wizard.focusOnEnter = function(event)
	{
		if (event.keyCode == 13)
		{
			var nextFocus = Doc.element(Data.get(event.currentTarget, 'next-focus'));
			if (nextFocus && nextFocus.focus) nextFocus.focus();
		}
	}
	
	// Event handler for the 'Next' navigation button
	wizard.goNextOnClick = function(event)
	{
		wizard.goNext(false);
	}
	
	/**
		Go to the next step in the wizard. This method will first
		submit 'wizard-validate' event to the current step. If at 
		least one event handler calls 'Event.preventDefault()' the 
		processing is aborted. 
		
		Otherwise, the current step is disabled and the 'wizard-initialize' 
		event is sent to the next step. The event handler is responsible
		for calling 'Wizard.setCurrentStep' to display the next step.
	*/
	wizard.goNext = function(auto)
	{
		var verify = Doc.element('verifyGoNext');
		if (verify && !verify.checked) return;
		
		var details = { 'auto': auto };
		
		// Send 'wizard-validate' event to current step
		var step = Doc.element('step-'+currentStep);
		var evt = document.createEvent('CustomEvent');
		evt.initCustomEvent('wizard-validate', true, true, details);
		if (!step.dispatchEvent(evt)) 
		{
			wizard.notifyError();
			return;
		}
	
		// Hide all navigation buttons and disable form controls in current step
		internal.hideButtons('navigation');
		internal.disableStep(step);
		
		// Send 'wizard-initialize' to the next step
		var nextStep = Doc.element('step-'+(currentStep+1));
		var evt = document.createEvent('CustomEvent');
		evt.initCustomEvent('wizard-initialize', true, true, details);
		nextStep.dispatchEvent(evt);
	}
	
	wizard.notifyError = function()
	{
		Doc.addClass('wizard', 'flash-error');
		setTimeout(internal.stopFlashing, 2000);
	}
	
	internal.stopFlashing = function()
	{
		Doc.removeClass('wizard', 'flash-error');
	}

	/**
		Finalize the registration of the wizard. This method will
		first submit 'wizard-validate' to the current step. If at 
		least one event handler calls 'Event.preventDefault()' the 
		processing is aborted.
		
		Otherwise, the current step is disabled and the 'wizard-submit' 
		event is sent to the 'wizard'. The event handler is responsible
		for finalizing the registration and displaying a proper
		message, for example, by calling Wizard.showFinalMessage()
		
		The 'data' is passed on to the 'wizard-validate' and 
		'wizard-submit' events as the Event.details object.
	*/
	wizard.goRegister = function(data)
	{
		// Check if verification is enabled
		var verify = Doc.element('verifyGoNext');
		if (verify && !verify.checked) return;
		
		// Send 'wizard-validate' event to current step
		var step = Doc.element('step-'+currentStep);
		var evt = document.createEvent('CustomEvent');
		evt.initCustomEvent('wizard-validate', true, true, data);
		if (!step.dispatchEvent(evt)) 
		{
			wizard.notifyError();
			return;
		}
	
		internal.hideButtons('navigation');
		internal.disableStep(step);
		
		// Send 'wizard-submit' to the next step
		var wiz = Doc.element('wizard');
		hasSentRegistration = true;
		var evt = document.createEvent('CustomEvent');
		evt.initCustomEvent('wizard-submit', true, true, data);
		wiz.dispatchEvent(evt);
	}
	
	wizard.setNoConfirmOnFirstStep = function(cf)
	{
		noConfirmOnFirstStep = cf;
	}
	
	// Ask the user for confirmation and then restart the wizard
	wizard.cancelWizard = function()
	{
		forceConfirm = true;
		wizard.restartWizard();
	}
	
	// Restart the wizard without asking the user for confirmation
	wizard.restartWizard = function()
	{
		var url = location.href;
		if (url.indexOf('&restart=1') == -1) url += '&restart=1';
		location.href = url;
	}
	
	wizard.setCurrentStep = function(stepNo)
	{
		currentStep = stepNo;
		internal.enableStep(Doc.element('step-'+currentStep));
	}

	internal.disableStep = function(step)
	{
		step = Doc.element(step);
		internal.disableAllFormElements(step);
		Doc.addClass(step, 'disabled');
		Doc.hide('gonext-message');
		var stepNo = step.getElementsByClassName('step-no')[0];
		Events.addEventHandler(stepNo, 'click', internal.togglePastStep, {'step-id': step.id});
		stepNo.title = 'Show/hide this section';
	}
	
	internal.enableStep = function(step)
	{
		step = Doc.element(step);
		Doc.removeClass(step, 'disabled');
		Doc.show(step);
	}
	
	// Toggle visibility of a past step
	// The step is hidden if the 'auto-hide' class is present
	internal.togglePastStep = function(event)
	{
		var step = Data.get(event.currentTarget, 'step-id');
		Doc.addOrRemoveClass(step, 'auto-hide');
	}

	
	/**
		A fatal error has occurred and the wizard will not be able to continue.
		This method will display the error message, disable all form input fields
		and buttons (except the 'Restart' button).
	*/
	wizard.setFatalError = function(err, messages)
	{
		hasFatalError = true;
		var msg = err;
		if (err.toString)
		{
			App.debug(err.toString());
			msg = Strings.encodeTags(err.toString());
			if (err.stack && !messages)
			{
				App.debug(err.stack);
				msg += '<br><pre>'+Strings.encodeTags(err.stack)+'</pre>';
			}
		}
		else
		{
			App.debug(msg);
		}
		
		if (messages) 
		{
			msg = '<div class="messagecontainer error">'+msg+'</div>'+internal.generateMessageList(messages).html;
			wizard.setWizardStatus('success', msg);
		}
		else
		{
			wizard.setWizardStatus('messagecontainer error', msg || 'Unexpected failure!');
		}
		
		Doc.addClass('wizard', 'disabled');
		internal.disableAllFormElements('wizard', true);
		Doc.show('gorestart');
	}

	
	/**
		Display a 'loading' global wizard-status message.
		The 'progress' parameter is optional. If given,
		it should be the name of a progress reporter.
		This activates the display of a progress bar which
		Relax will update at regular intervals in the background.
		See SessionServlet (cmd=GetProgress) for more information.
	*/
	wizard.showLoadingAnimation = function(msg, progress)
	{
		wizard.setWizardStatus('loading', msg  || 'Working, please wait...');
		if (progress)
		{
			var progressBar = Doc.element('wizard-progress');
			progressBar.classList.add('progress-bar');

			// Get/create child elements
			var done = Doc.element('wizard-progress-done');
			if (!done)
			{
				done = document.createElement('span');
				done.id = 'wizard-progress-done';
				done.className = 'progress-bar-done bg-filled-100 topborder bottomborder leftborder';
				progressBar.appendChild(done);
			}
			var remain = Doc.element('wizard-progress-remain');
			if (!remain)
			{
				remain = document.createElement('span');
				remain.id = 'wizard-progress-remain';
				remain.className = 'progress-bar-remain topborder bottomborder rightborder';
				progressBar.appendChild(remain);
			}
				
			progressBar.setAttribute('value', '0%');
			done.style.width = '0%';
			remain.style.width = '100%';
			Doc.show(progressBar);
			
			// Initial request for updating the progress bar
			progressTimer = setTimeout(internal.requestProgressUpdate, 1500, progress);
		}
	}
	
	/**
		Make a request for the progress information. The 'progress'
		parameter is the name of a progress bar.
	*/
	internal.requestProgressUpdate = function(progress)
	{
		if (hasFatalError) return;
		var url = '../Session.servlet?ID='+App.getSessionId();
		url += '&cmd=GetProgress';
		url += '&name='+encodeURIComponent(progress);
		
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, internal.onProgressReport, internal.onProgressReport);
		request.send(null);
	}
	
	/**
		Callback when receiving a progress update.
	*/
	internal.onProgressReport = function(request)
	{
		if (hasFatalError) return;
		if (debug) App.debug(Strings.encodeTags(request.responseText));
		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		if (error)
		{
			return;
		}
	
		var progress = response.progress;
		var progressBar = Doc.element('wizard-progress');
	
		if (!progress) 
		{
			// Give up if no progress information
			Doc.hide(progressBar);
			return;
		}
		
		var done = Doc.element('wizard-progress-done');
		var remain = Doc.element('wizard-progress-remain');
	
		progressBar.setAttribute('value', progress.percent+'%');
		done.style.width = progress.percent+'%';
		remain.style.width = (100-progress.percent)+'%';
		
		if (progress.message)
		{
			Doc.element('wizard-status').innerHTML = Strings.encodeTags(progress.message);
		}
		
		if (progress.percent < 100)
		{
			// Continue to request updates until we 100% has been reached
			progressTimer = setTimeout(internal.requestProgressUpdate, 1500, progress.name);
		}
		else
		{
			progressTimer = null;
			Doc.hide(progressBar);
		}
	}

	
	wizard.showStatusMessage = function(msg)
	{
		wizard.setWizardStatus('messagecontainer note', msg);
	}

	/**
		Hide the current global wizard-status message.
	*/
	wizard.hideWizardStatus = function()
	{
		Doc.hide('wizard-status');
		Doc.hide('wizard-progress');
	}
	
	wizard.showGoNextConfirmation = function(verify, msg)
	{
		var check = '';
		if (verify)
		{
			check = '<input type="checkbox" name="verifyGoNext" id="verifyGoNext">';
			Doc.addClass('gonext', 'disabled');
			Doc.addClass('goregister', 'disabled');
		}
		
		var gonext = Doc.element('gonext-message');
		gonext.innerHTML = check + msg;
		Doc.show(gonext);
		
		if (verify)
		{
			Events.addEventHandler('verifyGoNext', 'click', internal.verifyInternalOnClick);
		}
	}
	
	wizard.hideGoNextConfirmation = function()
	{
		var gonext = Doc.element('gonext-message');
		gonext.innerHTML = '';
		Doc.hide(gonext);
		Doc.removeClass('gonext', 'disabled');
		Doc.removeClass('goregister', 'disabled');
	}
	
	internal.verifyInternalOnClick = function(event)
	{
		var frm = document.forms['reggie'];
		var checked = event.currentTarget.checked;
		Doc.addOrRemoveClass('goregister', 'disabled', !checked);
		Doc.addOrRemoveClass('gonext', 'disabled', !checked);
	}

	/**
		Show final registration messages in a list.
	*/
	wizard.showFinalMessage = function(messages)
	{
		var msg = internal.generateMessageList(messages);

		var status = Doc.element('wizard-messages') || Doc.element('wizard-status');
		status.className = 'success';
		status.innerHTML = msg.html;
		Doc.show(status);

		if (progressTimer) clearTimeout(progressTimer);
		return { 'errors': msg.errors, 'warnings': msg.warnings };
	}

	internal.generateMessageList = function(messages)
	{
		var msg = '<ul>';
		var numWarnings = 0;
		var numErrors = 0;
		for (var i = 0; i < messages.length; i++)
		{
			var msgLine = messages[i];
			if (msgLine.indexOf('[Warning]') >= 0)
			{
				msg += '<li class="warning">' + Strings.encodeTags(msgLine.replace('[Warning]', ''));
				numWarnings++;
			}
			else if (msgLine.indexOf('[Error]') >= 0)
			{
				msg += '<li class="error">' + Strings.encodeTags(msgLine.replace('[Error]', ''));
				numErrors++;
			}
			else
			{
				msg += '<li class="ok">' + Strings.encodeTags(msgLine);
			}
		}
		msg += '</ul>';
		return { 'html': msg, 'errors': numErrors, 'warnings': numWarnings };
	}

	/**
		Initialize the wizard:
		
		* Check that a project is active
	*/
	internal.initWizard = function()
	{
		var path = document.getElementsByClassName('path');
		if (path && path.length)
		{
			var relaxVersion = document.createElement('span');
			relaxVersion.id = 'relax-version';
			relaxVersion.innerHTML = '[Relax '+Relax.VERSION+']';
			path[0].appendChild(relaxVersion);
		}
	}
		
	/**
		Display a global wizard-status message. This may for example be
		an error message, a work-in-progress indicator, etc.
	*/
	wizard.setWizardStatus = function(className, msg)
	{
		var status = Doc.element('wizard-status');
		status.className = className;
		status.innerHTML = msg;
		Doc.show(status);
	}

	// Disable all form elements inside the given container
	internal.disableAllFormElements = function(container, hideButtons)
	{
		container = Doc.element(container);
		if (!container) return;
		var input = container.getElementsByTagName('input');
		for (var i = 0; i < input.length; i++)
		{
			input[i].disabled = true;
		}
		var select = container.getElementsByTagName('select');
		for (var i = 0; i < select.length; i++)
		{
			select[i].disabled = true;
		}
		var text = container.getElementsByTagName('textarea');
		for (var i = 0; i < text.length; i++)
		{
			text[i].disabled = true;
		}
		if (hideButtons)
		{
			internal.hideButtons(container);
		}
	}

	// Hide all 'basicbuttons':s in the given container
	internal.hideButtons = function(container)
	{
		container = Doc.element(container);
		var buttons = container.getElementsByClassName('basicbutton');
		for (var i = 0; i < buttons.length; i++)
		{
			Doc.hide(buttons[i]);
		}
	}

	Doc.addFinalizer(internal.initWizard);
	return wizard;
}();

/**
	Holds one or more filter criterias. Use the 'add*' methods
	to add criterias to the filter, and the 'check(item)' method
	to evaluate all criterias against a single item.
*/
var Filter = function()
{
	this.criterias = [];
}

/**
	Adds a text criteria comparing the given item property to
	a static value (f2).
*/
Filter.prototype.addTextCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.eq;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a text criteria checking if the given item property to
	contains a static value (f2).
*/
Filter.prototype.addContainsCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.contains;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Evaluate all criterias. Return true if all pass, false otherwise.
*/
Filter.prototype.check = function(item)
{
	for (var i = 0; i < this.criterias.length; i++)
	{
		if (!this.criterias[i](item)) return false;
	}
	return true;
}


var Comparator = function()
{
	var cmp = {};
	
	/**
		Return a comparator based on the symbol
		which can be either '>' or '<' to return
		Comparator.gt or Comparator.lt. Otherwise
		the given default or Comparator.eq is returned.
	*/
	cmp.get = function(symbol, defaultCmp)
	{
		var c;
		if (symbol == '<')
		{
			c = cmp.lt;
		}
		else if (symbol == '>')
		{
			c = cmp.gt;
		}
		else
		{
			c = defaultCmp || cmp.eq;
		}
		return c;
	}
	
	// Compare if two values are equal
	cmp.eq = function(v1, v2)
	{
		return v1 == v2;
	}
	
	// Compare if two values are different
	cmp.neq = function(v1, v2)
	{
		return v1 != v2;
	}
	
	// Check if v2 is contained in v1
	cmp.contains = function(v1, v2)
	{
		return v1 != null && v2 != null && v1.indexOf(v2) >= 0;
	}
	
	// Compare if two values are appoximately equals (difference is not more than 0.05)
	cmp.approx = function(v1, v2)
	{
		return Math.abs(v2 - v1) < 0.05;
	}
	
	// Compare if v1 starts with the same string as v2
	cmp.startsWith = function(v1, v2)
	{
		return v1 != null && v1.indexOf(v2) == 0;
	}
	
	// Compare if v1 is greater than or equal to v2
	cmp.gt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 >= v2;
	}
	
	// Compare if v1 is less than or equal to v2
	cmp.lt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 <= v2;
	}
	
	return cmp;
}();



